import { assertInInjectionContext, inject, Injector, DestroyRef, isSignal } from '@angular/core';
import { takeUntilDestroyed, toSignal, toObservable } from '@angular/core/rxjs-interop';
import { share, Observable, filter, map } from 'rxjs';

/**
 * Create an `ObservableSignal` from:
 * - a synchronously `Observable`, or
 * - an asynchronous `Observable` with an initial value for the signal, or
 * - a `Signal`.
 */
function toObservableSignal(...args) {
    assertInInjectionContext(toObservableSignal);
    const injector = inject(Injector);
    const destroyRef = injector.get(DestroyRef);
    const processSourceObservable = (observable) => observable.pipe(share({ resetOnRefCountZero: false }), takeUntilDestroyed(destroyRef));
    let signal; // in fact Signal<Value | InitialValue>
    let observableFactory;
    if (args.length === 1 && args[0] instanceof Observable) {
        let observable = args[0];
        observable = processSourceObservable(observable);
        signal = toSignal(observable, { requireSync: true });
        observableFactory = () => observable;
    }
    else if (args.length === 2 && args[1] instanceof Observable) {
        const initialValue = args[0];
        let observable = args[1];
        observable = processSourceObservable(observable);
        signal = toSignal(observable, { initialValue });
        observableFactory = () => observable;
    }
    else if (args.length === 1 && isSignal(args[0])) {
        signal = args[0];
        observableFactory = () => toObservable(signal, { injector });
    }
    else {
        throw new Error('Invalid arguments');
    }
    let observable;
    return Object.assign(signal, {
        observe: () => {
            observable ??= observableFactory();
            return observable;
        },
    });
}

/**
 * RxJS operator that picks only the values that are instances
 * of the provided types.
 *
 * @example
 *  ```ts
 *  router.events.pipe(
 *    pickType(NavigationStart, NavigationEnd)
 *  ).subscribe(console.log);
 *  ```
 */
const pickType = (...types) => (source) => source.pipe(filter((c) => types.some((t) => c instanceof t)));
/**
 * RxJS operator that maps values to `undefined`.
 * Useful when an `Observable<void>` type is expected.
 *
 * @remarks Shorthand for `map(() => undefined)`.
 *
 * @example
 * ```ts
 * const void$: Observable<void> = router.events.pipe(
 *   tap((v) => console.log(v)),
 *   mapToVoid(),
 * );
 * ```
 */
const mapToVoid = () => map(() => undefined);

/**
 * Generated bundle index. Do not edit.
 */

export { mapToVoid, pickType, toObservableSignal };
//# sourceMappingURL=angularity-core-rxjs.mjs.map
