import { inject, Injector, runInInjectionContext, PLATFORM_ID } from '@angular/core';
import { isPlatformBrowser } from '@angular/common';
import * as upstream from 'type-container';

/**
 * An Exception is a runtime exception that is meant to caught and handled.
 * On the other hand, an Error is unrecoverable and are not meant to be
 * caught.
 *
 * @example
 * ```ts
 * export class IndexOutOfRangeException extends Exception {
 *   constructor(index: number) {
 *     super(`Index ${index} is out of range`);
 *   }
 * }
 * ```
 *
 * @remarks Extending this class will automatically override the `name`
 * property to the class name, resulting in a more descriptive error message.
 */
class Exception extends Error {
    constructor() {
        super(...arguments);
        this.name = this.constructor.name;
    }
}

/**
 * Inject a proxy of the target dependency, which defers the actual injection
 * until any of its properties is accessed.
 *
 * @remarks
 * This allows circular dependency injection (even though you ought not to),
 * as long as the two classes do not access each other's properties during
 * construction.
 *
 * @example
 *  ```ts
 *  export class MyService {
 *    private dep = injectRef(AnotherService);
 *  }
 *  ```
 *  ```ts
 *  export class AnotherService {
 *    private dep = injectRef(MyService);
 *  }
 *  ```
 */
function injectRef(token, injector = inject(Injector)) {
    let instance;
    return new Proxy({}, {
        get(_, prop) {
            if (!instance)
                instance = injector.get(token);
            return instance[prop];
        },
    });
}
/**
 * Inject a service, lazily loaded as a promise.
 *
 * @param token promise of the token to inject
 * @param injector injector to use to instantiate the dependency
 * @returns promise of the service instance
 *
 * @example
 * ```ts
 *  private myService = injectLazy(import('./my-service').then(m => m.MyService));
 * ```
 * ```ts
 *  async someMethod() {
 *    const service = await this.myService;
 *    service.doSomething();
 *  }
 * ```
 */
async function injectLazy(token, injector = inject(Injector)) {
    return token.then((t) => injector.get(t));
}
/**
 * Return a function that accepts another function.
 * The accepted function will be executed synchronously in the
 * current injection context, and its return value will be forwarded.
 *
 * @example
 * ```ts
 *  private inContext = useInjectionContext();
 * ```
 * ```ts
 * const result = inContext(() => inject(MyService).doSomething());
 * ```
 */
const useInjectionContext = (injector = inject(Injector)) => (fn) => runInInjectionContext(injector, fn);

/**
 * Return a `Executor` if the current platform matches the given
 * `PlatformSelector`, or `null` otherwise.
 *
 * @example
 *  ```ts
 *  #onBrowser = usePlatformExec(isPlatformBrowser);
 *  ```
 *  ```ts
 *  if (!this.#onBrowser) return;
 *  doSomethingServerSpecific();
 *  ```
 *  ```ts
 *  // href is `undefined` if not in a browser environment
 *  const href = this.onBrowser?.(() => window.location.href);
 *  ```
 *  ```ts
 *  // alert is only invoked in a browser environment
 *  this.onBrowser?.(() => alert('Hello, World!'));
 *  ```
 */
function usePlatformExec(selector) {
    const platformId = inject(PLATFORM_ID);
    if (!selector(platformId))
        return null;
    return (fn) => fn();
}
/**
 * Create a function that accepts another function.
 * If the current platform matches the given `PlatformSelector`, the accepted
 * function is invoked and its return value is forwarded.
 * Otherwise, `null` is returned.
 *
 * @deprecated prefer `usePlatformExec`.
 *
 * @example
 *  ```ts
 *  private onBrowser = usePlatformOnly(BROWSER);
 *  ```
 *  ```ts
 *  // href is `null` if not in a browser environment
 *  const href = this.onBrowser(() => window.location.href);
 *  ```
 *  ```ts
 *  // alert is only invoked in a browser environment
 *  this.onBrowser(() => alert('Hello, World!'));
 *  ```
 */
const usePlatformOnly = (selector, platformId = inject(PLATFORM_ID)) => (fn) => {
    if (!selector(platformId))
        return null;
    return fn();
};
/**
 * Dependency Assembler that assembles a function, which takes another function,
 * invokes it and forwards its return value if within a browser environment, or
 * return `null` otherwise.
 *
 * @deprecated Prefer `usePlatformOnly`.
 *
 * @example
 *  ```ts
 *  private browserOnly = useBrowserOnly();
 *  ```
 *  ```ts
 *  // href is `null` if not in a browser environment
 *  const href = this.browserOnly(() => window.location.href);
 *  ```
 *  ```ts
 *  // alert is only invoked in a browser environment
 *  this.browserOnly(() => alert('Hello, World!'));
 *  ```
 */
const useBrowserOnly = (platform = inject(PLATFORM_ID)) => (fn) => {
    if (!isPlatformBrowser(platform))
        return null;
    return fn();
};

/**
 * Type-safe factory function for producing NON-multi `Provider` configs.
 * @param config type-safe configuration object
 * @returns a `Provider` object
 *
 * @see `provideMulti`
 *
 * @example
 *  ```ts
 *  provide({ token: TitleStrategy, useClass: AppTitleStrategy });
 *  provide({ token: TitleStrategy, useExisting: AppTitleStrategy });
 *  // `AppTitleStrategy` have to be assignable to `TitleStrategy`.
 *
 *  declare const SNACKBAR_CONFIG: InjectionToken<SnackbarConfig>;
 *  provide({ token: SNACKBAR_CONFIG, useValue: { ... } });
 *  // Type of `useValue` is inferred as `SnackbarConfig`, enabling autocomplete.
 *
 *  declare const BREAKPOINTS: InjectionToken<BreakpointMap>
 *  provide({ token: BREAKPOINTS, useFactory: (observer = inject(BreakpointObserver)) => observer.observe() });
 *  // Return type of the factory has to match `BreakpointMap`
 *
 *  provide({ token: APP_INITIALIZER, useFactory: () => () => {} });
 *  // Type error. APP_INITIALIZER should be an array of functions, but the return type of the factory is a single function.
 *  // Type '() => void' is not assignable to type 'readonly (() => void | Observable<unknown> | Promise<unknown>)[]'
 *  ```
 */
function provide(config) {
    if ('useValue' in config)
        return { provide: config.token, useValue: config.useValue };
    if ('useFactory' in config)
        return { provide: config.token, useFactory: config.useFactory };
    if ('useClass' in config)
        return { provide: config.token, useClass: config.useClass };
    if ('useExisting' in config)
        return { provide: config.token, useExisting: config.useExisting };
    throw new Error('Unhandled provide config');
}
/**
 * Type-safe factory function for producing multi `Provider` configs.
 * @param config type-safe configuration object
 * @returns a `Provider` object
 *
 * @see `provide`
 *
 * @example
 *  ```ts
 * provideMulti({ token: TitleStrategy, useClass: AppTitleStrategy });
 * // Type error. TitleStrategy does not support multi providing.
 * // Argument of type '{ token: typeof TitleStrategy; useClass: typeof AppTitleStrategy; }' is not assignable to parameter of type 'never'.
 *
 * provideMulti({ token: APP_INITIALIZER, useFactory: () => () => {} });
 * // Legit. The return type of the factory matches the item type of APP_INITIALIZER
 *  ```
 */
function provideMulti(config) {
    return {
        ...provide(config),
        multi: true,
    };
}

/* eslint-disable import/namespace */
/**
 * Forwarded symbol from upstream `type-container` for convenience.
 * @see https://github.com/Char2sGu/type-container
 */
const $type = upstream.$type;

/**
 * Generated bundle index. Do not edit.
 */

export { $type, Exception, injectLazy, injectRef, provide, provideMulti, useBrowserOnly, useInjectionContext, usePlatformExec, usePlatformOnly };
//# sourceMappingURL=angularity-core.mjs.map
